# -*- coding: utf-8 -*-
"""
    femagtools.mcv
    ~~~~~~~~~~~~~~

    Reading, Creating and managing MCV/MC files

    :copyright: 2016 Semafor Informatik & Energie AG, Basel
    :license: BSD, see LICENSE for more details.
"""
import json
import subprocess
import sys
import logging
import os.path
import struct
import math
import numpy as np
import femagtools.losscoeffs

logger = logging.getLogger(__name__)

transl = dict(
    cversion='version_mc_curve',
    desc='mc1_title',
    
    recalc='mc1_recalc',
    remz='mc1_remz',
    ctype='mc1_type',
    rho='mc1_fe_spez_weigth',
    ch='mc1_ch_factor',
    cw='mc1_cw_factor',
    ch_freq='mc1_ch_freq_factor',
    cw_freq='mc1_cw_freq_factor',
    fillfac='mc1_fillfac',
    fillfac_old='mc1_fillfac_old',
    bref='mc1_bref',
    bsat='mc1_bsat',
    Bo='mc1_base_induction',
    b_coeff='mc1_induction_factor',
    fo='mc1_base_frequency',
    a='mc1_a',
    b='mc1_b',
    hi='mc1_hi',
    nuer='mc1_nuer',
    bi='mc1_bi',
    bi2='mc1_bi2',
    db2='mc1_db2',
    fe_sat_mag='mc1_fe_sat_magnetization'
    )
    
loss_transl = dict(
    f='frequency',
    B='induction',
    pfe='losses',
    cw='cw_m', 
    alfa='alfa_m',
    beta='beta_m', 
    fo='base_freq', 
    Bo='base_induct'
)

MC1_MIMAX = 50
MC1_NIMAX = 50
M_LOSS_INDUCT = 20
M_LOSS_FREQ = 20

MUE0 = 4e-7*np.pi  # 1.2566371E-06


def findNotNone(l):
    """return lower and upper indexes of not none values in list"""
    for i in range(len(l)):
        if l[i]:
            break
    for j in range(len(l)-1, -1, -1):
        if l[j]:
            break
    return (i, j)


class Reader:
    def __init__(self):
        # default values from file: mcv.par
        self.ACT_VERSION_MC_CURVE = 0
        self.ORIENTED_VERSION_MC_CURVE = 1
        self.PARAMETER_PM_CURVE = 2
        self.DEMCRV_BR = 4

        self.MC1_BASE_FREQUENCY = 50.0
        self.MC1_BASE_INDUCTION = 1.5
        self.MC1_CH_FACTOR = 0.0
        self.MC1_CW_FACTOR = 0.0
        self.MC1_CH_FREQ_FACTOR = 0.0
        self.MC1_CW_FREQ_FACTOR = 0.0
        self.MC1_INDUCTION_FACTOR = 0.0
        self.MC1_FE_SPEZ_WEIGTH = 7.65
        self.MC1_FE_SAT_MAGNETIZATION = 2.15

        self.mc1_title = ''
        self.version_mc_curve = self.ACT_VERSION_MC_CURVE
        self.mc1_type = 1
        self.mc1_remz = 0.0
        self.mc1_recalc = 0
        self.mc1_bsat = 0.0
        self.mc1_bref = 0.0
        self.mc1_curves = 1

        self.mc1_fillfac = 1.0
        self.rho = 7.6
        self.MCURVES_MAX = 20
        self.MC1_NIMAX = 50
        self.MC1_MIMAX = 50

        self.curve=[]
        self.mc1_mi    = [ 0. for I in range(self.MCURVES_MAX)  ]
        self.mc1_db2   = [ 0. for I in range(self.MCURVES_MAX)  ]
        self.mc1_angle = [ 0. for I in range(self.MCURVES_MAX)  ]

        self.mc1_ni = [ float('nan') for I in range(self.MCURVES_MAX)  ]

        self.mc1_bi=[ [ float('nan') for I in range(self.MCURVES_MAX)  ] for K in range(self.MC1_NIMAX) ]
        self.mc1_hi=[ [ 0. for I in range(self.MCURVES_MAX)  ] for K in range(self.MC1_NIMAX) ]

        self.mc1_bi2=[ [ 0. for I in range(self.MCURVES_MAX)  ] for K in range(self.MC1_MIMAX) ]
        self.mc1_nuer=[ [ 0. for I in range(self.MCURVES_MAX)  ] for K in range(self.MC1_MIMAX) ]
        self.mc1_a=[ [ 0. for I in range(self.MCURVES_MAX)  ] for K in range(self.MC1_MIMAX) ]
        self.mc1_b=[ [ 0. for I in range(self.MCURVES_MAX)  ] for K in range(self.MC1_MIMAX) ]
        self.mc1_energy=[ [ 0 for I in range(self.MCURVES_MAX)  ] for K in range(self.MC1_MIMAX) ]

    def getString(self, length=1):
        block = self.fp.read(length)
        st = []
        for i in range(0, len(block)):
            (s,) = struct.unpack('c', block[i:i+1])
            if ord(s) != 4 and ord(s) != 0  and ord(s) != 12  and ord(s) != 16 :
                st.append(s)
#        print "getString: [", "", st, "][", "", st.strip(), "] REAL[",real,"] len(", len(st), ') lenStrip(', len(st.strip()), ')\n'
        return b''.join(st).decode('latin1').strip().encode('utf-8')
        
    def getInteger(self):
        block = self.fp.read(4)
        (integer,) = struct.unpack('i', block[0:4])
#        print "Integer: ", integer
        return integer

    def rtrimValueList(self, vlist):
        le = len(vlist)
        for i in range(le-1, -1, -1):
            if vlist[i] == 0.:
                vlist = vlist[:-1]
            else:
                break
        return vlist
  
    def getReal(self):
        block = self.fp.read(4)
        if len(block) == 4 and ord(block[0:1]) in [12, 16] and \
               ord(block[1:2]) == 0 and ord(block[2:3]) == 0 and \
               ord(block[3:4]) == 0:
#            print "Read Next "
            return self.getReal()
#        for i in range(0, len(block)):
#            (s,) = struct.unpack('c', block[i:i+1])
#            print "RE C:", ord(s)
        if len(block) == 4:
            (real,) = struct.unpack('f', block[0:4])
#            print "Real: ", real
            return real
        else:
            return float('nan')
 
    def readMcv(self, filename):
        # intens bug : windows needs this strip to remove '\r'
        filename = filename.strip()

        if filename.upper().endswith('.MCV') or \
           filename.upper().endswith('.MC'):
            binary = True
            self.fp = open(filename, "rb")
        else:
            binary = False
            self.fp = open(filename, "r")

        self.name = os.path.splitext(filename)[0]
        # read curve version (INTEGER)
        if binary:
            str = self.getString(4)  # dummy 4 '\4\0\0\0'
            self.version_mc_curve = self.getInteger()
            str = self.getString(8)  # dummy 8 '\4\0\0\0' + '(' + '\0\0\0'
        else:
            self.version_mc_curve = int(self.fp.readline().strip())
        logger.info("MC Version %s", self.version_mc_curve)
        
        # read dummy text and title 2x (CHARACTER*40)
        if binary:
            str = self.getString(40)  # info text '*** File with magnetic curve ***'
            str = self.getString(8)  # dummy 8 '(('
            self.mc1_title = self.getString(40)  # read title
        else:
            self.fp.readline().strip()
            self.mc1_title = self.fp.readline().strip()
            
        #self.mc1_title = self.mc1_title.encode('utf-8')

        #read line 4
        if binary:
            str = self.getString(8) # dummy 8 '('+'\0\0\0\20\0\0\0'
            self.mc1_ni[0]  = self.getInteger()  # mc1_ni (INTEGER)
            self.mc1_mi[0]  = self.getInteger()  # mc1_mi (INTEGER)
            self.mc1_type   = self.getInteger()  # mc1_type (INTEGER)
            self.mc1_recalc = self.getInteger()  # mc1_recalc (INTEGER)
            self.mc1_db2[0] = self.getReal()     # mc1_db2 (REAL)
        else:
            line = self.fp.readline().split()
            self.mc1_ni[0]  = int (line[0])   # mc1_ni (INTEGER)
            self.mc1_mi[0]  = int (line[1])   # mc1_mi (INTEGER)
            self.mc1_type   = int (line[2])   # mc1_type (INTEGER)
            self.mc1_recalc = int (line[3])   # mc1_recalc (INTEGER)
            self.mc1_db2[0] = float (line[4]) # mc1_db2 (REAL)

        #read line 5
        if binary:
            self.getString(8) # dummy 8
            self.mc1_remz = self.getReal()
            self.mc1_bsat = self.getReal()
            self.mc1_bref = self.getReal()
            self.mc1_fillfac = self.getReal()
            if self.version_mc_curve == self.ORIENTED_VERSION_MC_CURVE or \
                   self.version_mc_curve == self.PARAMETER_PM_CURVE:
                self.mc1_curves = self.getInteger()

        else:
            line = self.fp.readline()
            (self.mc1_remz, self.mc1_bsat, self.mc1_bref, self.mc1_fillfac) = \
                            map(float, line.split())
            
        if self.version_mc_curve == self.ORIENTED_VERSION_MC_CURVE or \
               self.version_mc_curve == self.PARAMETER_PM_CURVE:
            if binary:
                self.mc1_curves = self.getInteger()
            else:
                self.mc1_curves = int(line[4])

        if self.mc1_type == self.DEMCRV_BR :
            self.mc1_angle[mc1_curves] = self.mc1_remz

        if binary:
            self.getString(8) # dummy 8
        else:
            # read rest of file and convert all to float values
            values = map( float, ' '.join(self.fp.readlines()).split())

        self.curve=[{} for i in range(self.mc1_curves)]
        for K in range(0, self.mc1_curves):
            if binary:
                [mc_bi,mc_hi]=zip(*[(self.getReal(), self.getReal())
                                    for I in range(self.MC1_NIMAX) ])
#                self.getString(8) # dummy 8
                self.getString(8)
                [mc_bi2, mc_nuer]=zip(*[(self.getReal(), self.getReal())
                                       for I in range(self.MC1_MIMAX) ])
                self.getString(8) # dummy 8
                [mc_a,mc_b, mc_c, mc_d]=zip(*[(self.getReal(), self.getReal(),
                                               self.getReal(), self.getReal())
                                              for I in range(self.MC1_MIMAX) ])
                self.getString(8) # dummy 8
            else:
                [mc_bi,mc_hi]= zip(*[values[2*I:2*I+2] for I in range(self.MC1_NIMAX)])
                idxOffset = 2*self.MC1_NIMAX
                [mc_bi2,mc_nuer]= zip(*[values[idxOffset+2*I:idxOffset+2*I+2]
                                        for I in range(self.MC1_NIMAX)])
                idxOffset += 2*self.MC1_NIMAX
                [mc_a,mc_b, mc_c, mc_d]= zip(*[values[idxOffset+4*I:idxOffset+4*I+4]
                                               for I in range(self.MC1_NIMAX)])
                idxOffset += 4*self.MC1_NIMAX

            if self.version_mc_curve == self.ORIENTED_VERSION_MC_CURVE or \
                   self.version_mc_curve == self.PARAMETER_PM_CURVE :
                if binary:
                    (self.mc1_angle[K], self.mc1_db2[K]) = (self.getReal(), self.getReal())
                else:
                    (self.mc1_angle[K], self.mc1_db2[K])=  \
                                        (values[idxOffset:idxOffset+2])
                    idxOffset += 2


            self.curve[K]['hi'] = self.rtrimValueList(
                [ mc_hi[I] for I in range(self.MC1_NIMAX)  ] )
            self.curve[K]['bi'] = self.rtrimValueList(
                [ mc_bi[I] for I in range(self.MC1_NIMAX)  ] )
            
            for I in range(self.MC1_NIMAX):
                if mc_bi[I] != 0.0 or mc_hi[I] != 0.0 :
                    self.mc1_ni[K] = I+1

            self.curve[K]['bi2']  = self.rtrimValueList(
                [ mc_bi2[I] for I in range(self.MC1_MIMAX)  ] )

            self.curve[K]['nuer'] = self.rtrimValueList(
                [ mc_nuer[I] for I in range(self.MC1_MIMAX)  ] )
            self.curve[K]['a'] = self.rtrimValueList(
                [ mc_a[I] for I in range(self.MC1_MIMAX)  ] )
            self.curve[K]['b'] = self.rtrimValueList(
                [ mc_b[I] for I in range(self.MC1_MIMAX)  ] )
            
            for I in range(self.MC1_MIMAX):
                if mc_a[I] !=  0.0 or mc_b[I] != 0.0 : 
                    self.mc1_mi[K] = I+1

        # set dummy defaults
        vals = [ self.MC1_BASE_FREQUENCY,
                 self.MC1_BASE_INDUCTION,
                 self.MC1_CH_FACTOR,
                 self.MC1_CW_FACTOR,
                 self.MC1_CH_FREQ_FACTOR,
                 self.MC1_CW_FREQ_FACTOR,
                 self.MC1_INDUCTION_FACTOR,
                 self.MC1_FE_SPEZ_WEIGTH,
                 self.MC1_FE_SAT_MAGNETIZATION]

        #print vals

        if binary:
            self.getString(8) # dummy 8
            for I in range(9):
                f = self.getReal()
                if math.isnan(f):
                    break
#                print "set dummy I: ", I, "  value: ", f
                vals[I] = f
        else:
            iLen = min(int(s) for s in [9, (len(values)-idxOffset)])
            for I in range(iLen):
                vals[I] = values[idxOffset+I]
            idxOffset += iLen
     
        #print vals

        self.fo = vals[0]
        self.Bo = vals[1]
        self.ch = vals[2]
        self.ch_freq = vals[4]
        self.cw = vals[3]
        self.cw_freq = vals[5]
        self.b_coeff = vals[6]
        self.rho = vals[7]
        self.fe_sat_mag = vals[8]

        if self.MC1_INDUCTION_FACTOR > 2.0:
            self.MC1_INDUCTION_FACTOR = 2.0
        for K in range(0, self.mc1_curves):
            energy = []
            if binary:
                for I in range(9):
                    f = self.getReal()
                    if math.isnan(f):
                        break
                    energy.append(f)
            else:
                iLen = min([int(s)
                            for s in [self.MC1_MIMAX,
                                      (len(values)-idxOffset)]])
                for I in range(iLen):
                    energy.append(values[idxOffset+I])
                idxOffset += iLen
                
            self.mc1_energy[K] = [ energy[I] for I in range(len(energy))  ]

    def get_results(self):
#        print "version : ", self.version_mc_curve,
#        print "title : ", self.mc1_title
#        print "hi : ", self.mc1_hi
#        print "bi : ", self.mc1_bi
        result={
            'name':self.name,
            'desc': self.mc1_title.decode('utf-8'),
            'cversion': self.version_mc_curve,
            #                'ni': self.mc1_ni,
            #                'mi': self.mc1_mi,
            'ctype': self.mc1_type,
            'recalc': self.mc1_recalc,
            'db2': self.mc1_db2,
            'remz': self.mc1_remz,
            'bsat': self.mc1_bsat,
            'bref': self.mc1_bref,
            'fillfac': self.mc1_fillfac,
            ###                'curves': self.mc1_curves,
            'curve' : self.curve,
            'energy':self.mc1_energy,
            'fo':self.fo,
            'Bo': self.Bo,
            'ch': self.ch,
            'ch_freq': self.ch_freq,
            'cw':self.cw,
            'cw_freq':self.cw_freq,
            'b_coeff':self.b_coeff,
            'rho':self.rho,
            'fe_sat_mag':self.fe_sat_mag }
      
#        print "Title: [",  self.mc1_title, "] CurveVersion :", self.version_mc_curve
#        print "LINE 0: ", self.mc1_ni[0], ' ',  self.mc1_mi[0], ' ',  self.mc1_type, ' ',  self.mc1_recalc, ' ',  self.mc1_db2[0]
#        print "LINE 1: ", self.mc1_remz, ' ',  self.mc1_bsat, ' ',  self.mc1_bref, ' ',  self.mc1_fillfac, ' ',  self.mc1_curves

        return result


class MagnetizingCurve(object):
    def __init__(self, mcvpar):
        """initialize this object either from
          a list of mcv objects or
          a single mcv or
          a directory"""
        self.mcv = {}
        if isinstance(mcvpar, list):
            logger.info("MagnetizingCurve is list")
            for m in mcvpar:
                if 'id' in m:
                    self.mcv[str(m['id'])] = m
                elif 'name' in m:
                    self.mcv[m['name']] = m
                    
        elif isinstance(mcvpar, dict):
            logger.info("MagnetizingCurve is dict")
            if 'id' in mcvpar:
                self.mcv[str(mcvpar['id'])] = mcvpar
            elif 'name' in mcvpar:
                self.mcv[mcvpar['name']] = mcvpar

        # Do not use unicode in PYTHON 3 as all strings are sequences of Unicode
        elif isinstance(mcvpar, str) or isinstance(mcvpar, unicode):
            self.mcdirectory = os.path.abspath(mcvpar)
            logger.info("MC Dir %s", self.mcdirectory)
        else:
            raise Exception("unsupported parameter type "+str(type(mcvpar)))
            
    def find(self, id):
        """find mcv by id or name"""
        try:
            return self.mcv[id]['name']
        except ValueError as ex:
            pass  # not found
        except KeyError as ex:
            try:
                ext = '.MC' if sys.platform == 'win32' else '.MCV'
                filename = ''.join((id, ext))
                logger.debug("search file %s in %s", filename,
                             self.mcdirectory)
                if os.access(os.path.join(self.mcdirectory,
                                          filename), os.R_OK):
                    return id
            except:
                pass
        logger.debug("search by name %s", id)
        m = self.find_by_name(id)
        return m['name'] if m else None
    
    def find_by_name(self, name):
        """find mcv by name"""
        for k in self.mcv.keys():
            if self.mcv[k]['name'] == name:
                    return self.mcv[k]
            # not found
        return None

    def recalc(self):
        for m in self.mcv:
            curve=self.mcv[m]['curve'][0]
            mi=MC1_MIMAX-2
            dh = curve['hi'][-1]-curve['hi'][-2]
            db = curve['bi'][-1]-curve['bi'][-2]
            dmue_d=db/dh
            dmue = curve['bi'][-1]/curve['hi'][-1]
            db = 3e-2*curve['bi'][-1]
            n3=1.5

            curve['muer']=[b/MUE0/h for b,h in zip( curve['bi'],curve['hi'] )]
            
            # extend bh-curve into saturation
            while dmue_d>1.01*MUE0 and dmue>1.5*MUE0:
                dmue_d=MUE0 + (dmue_d-MUE0)/np.sqrt(n3)
                curve['bi'].append(curve['bi'][-1]+db)
                curve['hi'].append(curve['hi'][-1]+db/dmue_d)
                curve['muer'].append(curve['bi'][-1]/MUE0/curve['hi'][-1])
                n3+=0.2
                dmue = curve['bi'][-1]/curve['hi'][-1]

            self.mcv[m]['db2'] = (curve['bi'][-1]**2 - curve['bi'][0]**2)/(mi-1)
            nuek0 = (curve['hi'][1]-curve['hi'][0])/\
              (curve['bi'][1]-curve['bi'][0])
            for j1 in range(len(curve['bi'])):
                    bk02=curve['bi'][j1]**2
                    if bk02>0: break
            curve['nuer']=[MUE0*nuek0]
            curve['bi2']=[bk02]
            curve['a']=[]
            curve['b']=[]

            bk1=0.0
            while bk1<=curve['bi'][-1]:
                    bk12 = bk02 + self.mcv[m]['db2']
                    bk1=np.sqrt(bk12)
                    j=2
                    while j<len(curve['bi']) and bk1 <= curve['bi'][j]:
                           j+=1
                    j-=1
                    bdel = curve['bi'][j] - curve['bi'][j1]
                    c1 = (curve['hi'][j] - curve['hi'][j1])/bdel
                    c2 = curve['hi'][j1] - c1*curve['bi'][j1]

                    nuek1 = c1 + c2/bk1

                    curve['a'].append(MUE0*(bk12*nuek0 - bk02*nuek1)/self.mcv[m]['db2'])
                    curve['b'].append(MUE0*(nuek1 - nuek0)/self.mcv[m]['db2'])
                    nuek0 = nuek1
                    bk02 = bk12

                    curve['nuer'].append(MUE0*nuek1)
                    curve['bi2'].append(bk12)

            curve['a'].append(1.0)
            curve['b'].append(MUE0*curve['hi'][-1]-curve['bi'][-1])
            
    def writefile(self, name, directory='.', writeproc='mcvwriter'):
        """find magnetic curve by name or id and write binary file
        returns filename if found else None"""
        if not id:
            return None
        ext = '.MC' if sys.platform == 'win32' else '.MCV'
        mcv = self.find_by_name(name)
        if not mcv:
            filename = ''.join((name, ext))
            try:
                import shutil
                logger.info("Copy file %s", filename)
                shutil.copy(os.path.join(self.mcdirectory,
                                         filename), directory)
                return filename
            except shutil.SameFileError:
                return filename
            except:
                logger.error("MCV %s not found", str(filename))
            return None

        filename = ''.join((mcv['name'], ext))
        try:
            logger.info("create %s", str(filename))
            proc = subprocess.Popen([writeproc],
                                    cwd=directory,
                                    stdin=subprocess.PIPE,
                                    stderr=subprocess.PIPE,
                                    bufsize=1)

            inparams = ["&INPARAM"]
            for k in mcv.keys():
                if k == 'name':
                    inparams.append("filename='{}'".format(filename))
                elif k == 'desc':
                    inparams.append("mc1_title='{}'".format(mcv['desc']))
                elif k == 'curve':
                    inparams.append('mc1_curves={}'.format(len(mcv[k])))
                    for c in mcv['curve']:
                        for n in c.keys():
                            if n not in transl:
                                continue
                            inparams.append("{}=".format(transl[n]))
                            if type(c[n]) is list:
                                inparams.append(
                                    ','.join(map(str, c[n])))
                            else:
                                inparams.append(str(c[n]))

                    if 'energy' in mcv['curve'][0] and len(mcv['curve'][0]['energy']) > 0:
                        inparams.append("mc1_energy=")
                        inparams.append(','.join(map(str, mcv['curve'][0]['energy'])))

                elif k == 'losses':
                    # find start index
                    bstart = 0
                    for a in mcv[k]['pfe']:
                        for i in range(len(a)):
                            if a[i]:
                                if i > bstart:
                                    bstart = i
                                break
                    inparams.append('N_freq={}'.format(len(mcv[k]['f'])))
                    inparams.append('N_J_ind={}'.format(len(mcv[k]['B'])-bstart))
                    inparams.append("frequency=")
                    inparams.append(",".join(map(str, mcv[k]['f'])))
                    inparams.append("induction=")
                    inparams.append(",".join(map(str, mcv[k]['B'][bstart:])))
                    inparams.append("losses=")
                    pfeT= []
#                    flen=10
#                    blen=20
                    cw = mcv[k]['cw']
                    alfa = mcv[k]['alfa']
                    beta = mcv[k]['beta']
                    fo  =mcv[k]['fo']
                    Bo = mcv[k]['Bo']

                    pfe = []
                    lower = 0
                    # must replace all None by approx losses
                    for i in range(len(mcv['losses']['f'])):
                        f = mcv['losses']['f'][i]
                        if f > 0:
                            pfei = [p[i] if i<len(p) else None for p in mcv['losses']['pfe']]
                            m, n = findNotNone(pfei)
                            if m > lower: lower=m
                            if m <= n:
                                y = [ np.log10(p) for p in pfei[m:n+1] ]
                                x = [ np.log10(b/Bo) for b in mcv['losses']['B'][m:n+1]]
                                A = np.vstack([x, np.ones(len(x))]).T
                                beta, cw = np.linalg.lstsq(A, y)[0]
                                for j in range(n+1,len(pfei)):
                                    pfei[j] = 10**cw*(mcv['losses']['B'][j]/Bo)**beta

                                pfe.append(pfei)
                    pfe+=[[0]*M_LOSS_INDUCT]*(M_LOSS_FREQ-len(mcv['losses']['f']))

                    for r in pfe:
                        a=list(r[lower:]) + [0]*(M_LOSS_INDUCT-len(r[lower:]))
                        #must copy last and cut "None" (null)
                        for i in range(len(a)-1,0,-1):
                            if a[i]>0: break
                            if a[i-1]>0:
                                a[i]=a[i-1]
                                break
                        pfeT+=a
                        
                    inparams.append(','.join(map(str, pfeT)))
                    inparams.append("cw_m={}".format(mcv[k]['cw']))
                    inparams.append("alfa_m={}".format(mcv[k]['alfa']))
                    inparams.append("beta_m={}".format(mcv[k]['beta']))
                    inparams.append("base_freq={}".format(mcv[k]['fo']))
                    inparams.append("base_induct={}".format(mcv[k]['Bo']))
                    logger.info("has losses N_freq %d, N_J_ind %d",
                                len(mcv[k]['f']), len(mcv[k]['B']))

                    inparams.append("loss_values_data=T")
                elif k not in transl:
                    continue
                else:
                    inparams.append("{}=".format(transl[k]))
                    if type(mcv[k]) is list:
                        inparams.append(','.join(map(str, mcv[k])))
                    else:
                        inparams.append("{}".format(mcv[k]))

            inparams.append("/\n")
            proc.stdin.write(bytes('\n'.join(inparams).encode('latin-1')))
            proc.stdin.close()
        except OSError as e:
            logger.error("PATH {}\n CWD {}\n Prog {}\n{}\n".format(
                os.environ['PATH'], directory, writeproc, str(e)))
            e.args += writeproc
            raise e
        except IOError as e:
            logger.error("{} {}\n".format(mcv['name'], str(e)))
            return None
        except KeyError as e:
            logger.error("{} key {} not found\n".format(mcv['name'], str(e)))
            return None

        for l in proc.stderr:
            logger.error(l)
        
        return filename
    
    def fitLossCoeffs(self):
        for m in self.mcv:
            if 'losses' not in self.mcv[m]:
                continue
            losses = self.mcv[m]['losses']
            cw, alfa, beta = femagtools.losscoeffs.fitsteinmetz(
                losses['f'],
                losses['B'],
                losses['pfe'],
                self.mcv[m]['Bo'],
                self.mcv[m]['fo'])
            losses['cw'] = cw
            losses['alfa'] = alfa
            losses['beta'] = beta
            losses['Bo'] = self.mcv[m]['Bo']
            losses['fo'] = self.mcv[m]['fo']
            
if __name__ == "__main__":
    if len(sys.argv) == 2:
        filename = sys.argv[1]
    else:
        filename = sys.stdin.readline().strip()

    with open(filename) as file:
        mcvdata = json.load(file)
        
    mcv = MagnetizingCurve(mcvdata)
    mcv.writefile(mcvdata['name'], '.', 'cat')
    #mcv.recalc()
    #mcv.writefile(mcvdata['name'], '.','cat')




"""
  Multi objective Optimizer
  ~~~~~~~~~~~~~~~~~~~~~~~~~


  :copyright: (c) 2016 Semafor Informatik & Energie AG, Basel
  :license: BSD, see LICENSE for more details.

"""
from .population import Population, Individual
from .problem import Problem
from .algorithm import Nsga2



# -*- coding: utf-8 -*-
"""
    femagtools.femag
    ~~~~~~~~~~~~~~~~

    Running FEMAG

    :copyright: 2016 Semafor Informatik & Energie AG, Basel
    :license: BSD, see LICENSE for more details.
"""
import subprocess
import os
import glob
import logging
try:
    import zmq
except ImportError:
    pass
import sys
import json
import io
import femagtools.mcv
import femagtools.fsl
import femagtools.ntib as ntib
import femagtools.config as cfg
import time
import platform
import re

logger = logging.getLogger(__name__)

BCHEXT = '.BATCH' if sys.platform.startswith('linux') else '.BCH'  # win32


class FemagError(Exception):
    pass


class BaseFemag(object):
    def __init__(self, workdir, cmd, magnetizingCurves, magnets):
        self.workdir = workdir
        if cmd:
            self.cmd = cmd
        else:
            self.cmd = cfg.get_femag()

        if magnetizingCurves:
            if isinstance(magnetizingCurves,
                          femagtools.mcv.MagnetizingCurve):
                self.magnetizingCurves = magnetizingCurves
            else:
                self.magnetizingCurves = femagtools.mcv.MagnetizingCurve(
                    magnetizingCurves)
        else:
            self.magnetizingCurves = []
        if magnets:
            if isinstance(magnets, femagtools.Magnet):
                self.magnets = magnets
            else:
                self.magnets = femagtools.Magnet(magnets)
        else:
            self.magnets = []

    def copy_magnetizing_curves(self, model, dir=None):
        """extract mc names from model and write files into workdir or dir if given
        :returns: list of extracted mc names
        """
        dest = dir if dir else self.workdir
        return [self.magnetizingCurves.writefile(m, dest)
                for m in model.set_magcurves(self.magnetizingCurves)]

    def create_fsl(self, pmMachine, operatingConditions):
        """create list of fsl commands"""
        model = femagtools.MachineModel(pmMachine)
        self.modelname = model.name
        self.copy_magnetizing_curves(model)

        builder = femagtools.fsl.Builder()
        return builder.create(model, operatingConditions, self.magnets)

    def get_log_value(self, pattern, modelname='FEMAG-FSL.log'):
        result = []
        pat = re.compile(pattern)
        with open(os.path.join(self.workdir, 'FEMAG-FSL.log')) as f:
            for line in f:
                m = pat.search(line)
                if m:
                    result.append(float(line.split(':')[-1].split()[0]))
        return result

    def read_bch(self, modelname=None):
        "read most recent BCH/BATCH file and return result"
        # read latest bch file if any
        if not modelname:
            modelname = self._get_modelname_from_log()

        result = femagtools.bch.Reader()
        bchfile_list = sorted(glob.glob(os.path.join(
            self.workdir, modelname+'_[0-9][0-9][0-9]'+BCHEXT)))
        if len(bchfile_list) > 0:
            logger.info("Read BCH {}".format(bchfile_list[-1]))
            with io.open(bchfile_list[-1], encoding='latin1',
                         errors='ignore') as f:
                result.read(f)
        return result
    
    def read_los(self, modelname=None):
        "read most recent LOS file and return result"
        # read latest los file if any
        if not modelname:
            modelname = self._get_modelname_from_log()

        losfile_list = sorted(glob.glob(os.path.join(
            self.workdir, modelname+'_[0-9][0-9][0-9].LOS')))
        if len(losfile_list) > 0:
            return ntib.read_los(losfile_list[-1])

        return dict()

    def _get_modelname_from_log(self):
        """
        Read the modelname from the Femag Log file
        """
        with open(os.path.join(self.workdir, 'FEMAG-FSL.log')) as f:
            for l in f:
                if l.startswith('New model') or l.startswith('Load model'):
                    model = l.split()[2].replace('"', '').replace(',', '')
                    break
        return model


class Femag(BaseFemag):
    """Invoke and control execution of FEMAG

    :param workdir: name of working directory
    :param cmd: name of femag program
    :param magnetizingCurves: collection of lamination material curves
    :param magnets: collection of magnet material
    """
    def __init__(self, workdir, cmd=None,
                 magnetizingCurves=None, magnets=None):
        super(self.__class__, self).__init__(workdir, cmd,
                                             magnetizingCurves, magnets)

    def run(self, filename, options=['-b'], fsl_args=[]):
        """invoke FEMAG in current workdir

        :param filename: name of file to execute
        :param options: list of FEMAG options
        :param fsl_args: list of FSL argument options
        :raises: FemagError
        """
        if self.cmd.startswith('wfemag') and \
           '-b' in options and \
           '-m' not in options:
            options.insert(0, '-m')
        args = [self.cmd] + options + [filename] + fsl_args

        basename, ext = os.path.splitext(os.path.basename(filename))
        outname = os.path.join(self.workdir, basename+'.out')
        errname = os.path.join(self.workdir, basename+'.err')
        with open(outname, 'w') as out, open(errname, 'w') as err:
            logger.info('invoking %s', ' '.join(args))
            proc = subprocess.Popen(
                args,
                stdout=out, stderr=err, cwd=self.workdir)

        proc.wait()
        errs = []
        # print femag output
        print(outname)
        with io.open(outname, encoding='latin1', errors='ignore') as outfile:
            for l in outfile:
                print(l.strip())
                if l.find('ERROR') > -1:
                    errs.append(l.strip())

        rc = proc.returncode
        logger.info("%s exited with returncode %d (num errs=%d)",
                    self.cmd, rc, len(errs))
        if rc != 0 or errs:
            with io.open(errname, encoding='latin1',
                         errors='ignore') as errfile:
                for l in errfile:
                    errs.append(l.strip())
            errs.insert(0, 'Exit code {}'.format(rc))
            raise FemagError(errs)
        
    def cleanup(self):
        "removes all created files in workdir"
        if not os.path.exists(self.workdir):
            return
        cleanfiles = ('*.B*CH', '*.I*7-*', '*.A*7-*',
                      '*.dat', '*.LOS', '*.svg', '*.png')
        # '*.TMC','*.TMO', '*.PROT', '*.hxy'):
        for p in cleanfiles:
            for f in glob.glob(os.path.join(self.workdir, p)):
                os.remove(f)

    def __call__(self, pmMachine, operatingConditions):
        """setup fsl file, run calculation and return BCH results"""
        fslfile = 'femag.fsl'
        with open(os.path.join(self.workdir, fslfile), 'w') as f:
            f.write('\n'.join(self.create_fsl(pmMachine,
                                              operatingConditions)))
        if operatingConditions['calculationMode'] == "pm_sym_loss":
            with open(os.path.join(self.workdir,
                                   self.modelname+'.ntib'), 'w') as f:
                f.write('\n'.join(ntib.create(
                    operatingConditions['speed'],
                    operatingConditions['current'],
                    operatingConditions['angl_i_up'])))
                # TODO: add r1, m

        self.run(fslfile)
        if operatingConditions['calculationMode'] == "pm_sym_loss":
            return self.read_los(self.modelname)
        return self.read_bch(self.modelname)


class ZmqFemag(BaseFemag):
    """Invoke and control execution of FEMAG with ZeroMQ

    :param workdir: name of working directory
    :param cmd: name of femag program
    """
    def __init__(self, workdir, port, host='localhost', cmd=None,
                 magnetizingCurves=None, magnets=None):
        super(self.__class__, self).__init__(workdir, cmd,
                                             magnetizingCurves, magnets)
        self.host = host
        self.port = port
        self.request_socket = None
        self.subscriber_socket = None

    def __del__(self):
        if self.request_socket:
            self.request_socket.close()
        if self.subscriber_socket:
            self.subscriber_socket.close()

    def __req_socket(self):
        """returns a new request client"""
        if self.request_socket:
            return self.request_socket
        context = zmq.Context.instance()
        self.request_socket = context.socket(zmq.REQ)
        self.request_socket.connect('tcp://{0}:{1}'.format(
            self.host, self.port))
        self.request_socket.setsockopt(zmq.LINGER, 500)
        return self.request_socket

    def __sub_socket(self):
        """returns a new subscriber client"""
        context = zmq.Context.instance()
        if self.subscriber_socket:
            return self.subscriber_socket
        self.subscriber_socket = context.socket(zmq.SUB)
        self.subscriber_socket.connect(
            'tcp://{0}:{1}'.format(
                self.host, self.port+1))
        self.subscriber_socket.setsockopt(zmq.SUBSCRIBE, b'')
        return self.subscriber_socket

    def __is_process_running(self, procId):
        try:
            import psutil
            return psutil.pid_exists(procId)
        except:
            pass
        # long version, self made
        try:
            if procId > 0:
                if platform.system() == "Windows":
                    #                   if procId in psutil.get_pid_list():
                    proc = subprocess.Popen(["tasklist"],
                                            stdout=subprocess.PIPE)
                    for l in proc.stdout:
                        ls = l.split()
                        try:
                            if str(procId) == ls[1]:
                                return True
                        except:
                            continue
                else:
                    if not os.kill(procId, 0):
                        return True
        except OSError as e:
            # No such process
            logger.info("OSError: '{}'\n".format(str(e)))
            return False
        except Exception:
            # we cannot check processId
            logger.info("Error: unknown\n")
            return True
        return False

    def __is_running(self, timeout=1500):
        """check if FEMAG is running in ZMQ mode

        :param timeout: The timeout (in milliseconds) to wait for a response
        :returns: True if FEMAG is running, False otherwise

        """
        try:
            request_socket = self.__req_socket()
            request_socket.send_string("FSL", flags=zmq.SNDMORE)
            request_socket.send_string("testvar=0")
            poller = zmq.Poller()
            # use POLLIN for recv, POLLOUT for send
            poller.register(request_socket, zmq.POLLIN)
            if poller.poll(timeout):  # ok, femag is running
                logger.info('femag is running for %s',
                            self.request_socket.getsockopt_string(
                                zmq.IDENTITY))
                request_socket.recv_multipart()
                return True
        except Exception as e:
            logger.error(e)
        self.request_socket.close()
        self.request_socket = None
        logger.debug("femag is not running")
        return False

    def send_fsl(self, fsl, header='FSL'):
        """sends FSL commands in ZMQ mode and blocks until commands are processed

        :param fsl: FSL commands
        :returns: response
        """
        try:
            request_socket = self.__req_socket()
            request_socket.send_string(header, flags=zmq.SNDMORE)
            request_socket.send_string(fsl)
            response = request_socket.recv_multipart()
            logger.debug("send_fsl["+fsl+"] done")
            return [s.decode() for s in response]
        except Exception as e:
            logger.error("send_fsl, for error: %s", str(e))
        return ['{"status":"error"}']

    def read(self):
        """reads from subscriber socket"""
        subscriber_socket = self.__sub_socket()
        try:
            response = subscriber_socket.recv_multipart()
            return [s.decode() for s in response]
        except Exception as e:
            logger.error(e)
            time.sleep(1)
        if self.subscriber_socket:
            self.subscriber_socket.close()
        self.subscriber_socket = None
        return None

    def run(self, options=['-b'], restart=False, procId=None):  # noqa: C901
        """invokes FEMAG in current workdir and returns pid

        :param options: list of FEMAG options
        :raises: FemagError
        """
        args = [self.cmd] + options

        if self.__is_running():
            if restart:
                logging.info("must restart")
                self.quit(True)

                # check if process really finished (mq_connection)
                logger.info("procId: %s", procId)
                if procId:
                    for t in range(200):
                        time.sleep(0.1)
                        if not self.__is_process_running(procId):
                            break
                        logger.info("femag (pid: '{}') not stopped yet".format(procId))
                logger.info("Stopped procId: %s", procId)
            else:
                try:
                    with open(os.path.join(self.workdir,
                                           'femag.pid'), 'r') as pidfile:
                        procId = int(pidfile.readline())
                except Exception:
                    pass
                return procId

        if self.request_socket:
            self.request_socket.close()
        self.request_socket = None

        basename = str(self.port)
        args.append(basename)

        outname = os.path.join(self.workdir, basename+'.out')
        errname = os.path.join(self.workdir, basename+'.err')
        with open(outname, 'w') as out, open(errname, 'w') as err:
            logger.info('invoking %s', ' '.join(args))
            proc = subprocess.Popen(
                args,
                stdout=out, stderr=err, cwd=self.workdir)

        # check if mq is ready for listening
        for t in range(200):
            time.sleep(0.1)
            if self.__is_running():
                logger.info("femag (pid: '{}') is listening".format(proc.pid))
                break

        # write femag.pid
        logger.info("ready %s", proc.pid)
        with open(os.path.join(self.workdir, 'femag.pid'), 'w') as pidfile:
            pidfile.write("{}\n".format(proc.pid))
        return proc.pid

    def quit(self, save_model=False):
        """terminates femag"""

        if not self.__is_running():
            return

        # send exit flags
        self.__req_socket()
        f = '\n'.join(['exit_on_end = true',
                       'exit_on_error = true'])
        response = self.send_fsl(f)

        # send quit command
        response = self.send_fsl('quit', 'CONTROL')

        # if query, send a answer
        obj = json.loads(response[0])
        if obj['status'] == 'Query':
            logger.info('query: %s => %s',
                        obj['message'], 'saved' if save_model else 'not saved')
            response = self.send_fsl('Ok' if save_model else 'Cancel')
        return response

    def __call__(self, pmMachine, operatingConditions):
        """setup fsl file, run calculation and return BCH results"""
        self.send_fsl('\n'.join(self.create_fsl(pmMachine,
                                                operatingConditions)))
        return self.read_bch(self.modelname)



# -*- coding: utf-8 -*-
"""
    femagtools.losscoeffs
    ~~~~~~~~~~~~~~~~~~~~~

    Fitting methods for loss coeffs

    :copyright: 2016 Semafor Informatik & Energie AG, Basel
    :license: BSD, see LICENSE for more details.
"""
import numpy as np
import scipy.optimize as so


def fitsteinmetz(f, B, losses, Bo, fo):
    """fit coeffs of 
        losses(f,B)=cw*(f/fo)**alfa*(B/Bo)**beta
       returns (cw, alfa, beta)
    """
    z = []
    for i, fx in enumerate(f):
        z += [(fx, bx, y) for bx, y in zip(B, np.array(losses).T[i])
              if isinstance(y, float)]
    
    fbx = np.array(z).T[0:2]
    y = np.array(z).T[2]

    steinmetz = lambda x, cw, alpha, beta: (
        cw*(x[0]/fo)**alpha*(x[1]/Bo)**beta)

    fitp, cov = so.curve_fit(steinmetz,
                             fbx, y, (1.0, 1.0, 2.0))
    return fitp


def fitjordan(f, B, losses, Bo, fo):
    """fit coeffs of 
      losses(f,B)=(cw*(f/fo)**alpha + ch*(f/fo)**beta)*(B/Bo)**gamma
    returns (cw, alpha, ch, beta, gamma)
    """
    z = []
    for i, fx in enumerate(f):
        if isinstance(B[0], float):
            z += [(fx, bx, y) for bx, y in zip(B, np.array(losses).T[i])
                  if y]
        else:
            z += [(fx, bx, y) for bx, y in zip(B[i], np.array(losses).T[i])
                  if y]

    jordan = lambda x, cw, alpha, ch, beta, gamma: (
        (cw*(x[0]/fo)**alpha +
         ch*(x[0]/fo)**beta) *
        (x[1]/Bo)**gamma)

    fbx = np.array(z).T[0:2]
    y = np.array(z).T[2]
    fitp, cov = so.curve_fit(jordan,
                             fbx, y, (1.0, 2.0, 1.0, 1.0, 1.0))
    return fitp


if __name__ == "__main__":

    f = [100.0, 200.0, 400.0, 1000.0, 2000.0]
    pfe = [[0.15, 0.34, 0.67, 1.87, 4.37],
           [0.37, 0.76, 1.66, 4.72, 10.84],
           [0.62, 1.28, 2.78, 8.42, 19.27],
           [1.08, 2.24, 4.91, 14.9, 34.67],
           [2.03, 4.28, 9.32, 28.12, 69.33],
           [3.46, 7.29, 15.72, 47.91, 118.7]]
    B = [0.3, 0.5, 0.7, 1.0, 1.5, 2.0]

    print(fitjordan(f, B, pfe, 1.5, 50.))
    print(fitsteinmetz(f, B, pfe, 1.5, 50.))



"""
    femagtools.tks
    ~~~~~~~~~~~~~~

    Manage TKS magnetizing curve data files

    :copyright: 2016 Semafor Informatik & Energie AG, Basel
    :license: BSD, see LICENSE for more details.
"""
import sys
import numpy as np
import os
import re
import codecs
import femagtools.losscoeffs
import json
import logging

logger = logging.getLogger(__name__)

MUE0 = 4e-7*np.pi  # 1.2566371E-06
fo = 50.
Bo = 1.5
numPattern = re.compile(r'([+-]?\d+(?:\.\d+)?(?:[eE][+-]\d+)?)')

MC1_MIMAX = 50
MC1_NIMAX = 50


def readlist(f):
    x = []
    while True:
        l = f.readline().strip()
        if not l:
            return list(zip(*x))
        vals = [float(n) for n in numPattern.findall(l.replace(',', '.'))]
        if vals:
            x.append(vals)


def pfe1(f, B, ch, fh, cw, fw, fb):
    return (ch*(f/fo)**fh + cw*(f/fo)**fw)*(B/Bo)**fb


def pfe2(f, B, cw, fw, fb):
    return cw*(f/fo)**fw * (B/Bo)**fb


class Reader:

    def __init__(self, filename):
        self.version_mc_curve = 0
        self.mc1_type = 1
        self.mc1_remz = 0.0
        self.mc1_recalc = 0
        self.mc1_bsat = 0.0
        self.mc1_bref = 0.0
        self.mc1_curves = 1

        self.mc1_fillfac = 1.0
        self.rho = 7.6
        self.MCURVES_MAX = 20
        self.MC1_NIMAX = 50
        self.MC1_MIMAX = 50
        self.mc1_db2 = [0. for I in range(self.MCURVES_MAX)]
#        self.mc1_energy = [[0 for I in range(self.MCURVES_MAX)]
#                           for K in range(self.MC1_MIMAX)]

        self.curve = []
        self.name = os.path.splitext(
            os.path.basename(filename))[0]
        self.mc1_title = 'ThyssenKrupp Steel'
        self.mc1_curves = 1
        self.curve = [{}]
        self.curve[0]['hi'] = []
        self.curve[0]['bi'] = []
        self.fo = fo
        self.Bo = Bo
        self.ch = 0.0
        self.ch_freq = 0.0
        self.cw = None
        self.cw_freq = None
        self.b_coeff = None
        self.rho = 7.6
        self.fe_sat_mag = None
        self.losses = dict(f=[], B=[], pfe=[])

        with codecs.open(filename, encoding='utf-8', errors='ignore') as f:
            while True:
                l = f.readline().strip()
                if not l:
                    if not self.losses['B']:
                        self.losses = dict()
                    break
                
                if l.startswith('H(A/m)	B(T)') or l.startswith('H[A/m]	B[T]'):
                    h, b, j = readlist(f)
                    self.curve[0]['hi'] = h
                    self.curve[0]['bi'] = b
                    
                elif l.startswith('Comment'):
                    self.mc1_title = l.split(':')[1].strip()
                        
                elif l.startswith('Mass Density'):
                    d = numPattern.findall(l.replace(',', '.'))
                    self.rho = float(d[0])
                    if l.split()[-1] == 'kg/m^3':
                        self.rho /= 1e3
                        
                elif l.startswith('f='):
                    fref = numPattern.findall(l.replace(',', '.'))
                    self.losses['f'].append(float(fref[0]))
                    b, p = readlist(f)
                    self.losses['B'].append(b)
                    self.losses['pfe'].append(p)

        if self.losses and not isinstance(self.losses['B'][0], float):
            import scipy.interpolate as ip
            z = femagtools.losscoeffs.fitjordan(
                self.losses['f'],
                self.losses['B'],
                self.losses['pfe'],
                self.Bo,
                self.fo)
            logger.info("Loss coeffs %s", z)
            self.ch = z[2]
            self.ch_freq = z[3]
            self.cw = z[0]
            self.cw_freq = z[1]
            self.b_coeff = z[4]

            # must normalize pfe matrix:
            bmin = max(list(zip(*(self.losses['B'])))[0])
            bmax = max([bx[-1] for bx in self.losses['B']])
            Bv = np.arange(np.ceil(10*bmin)/10.0,
                           (np.floor(10*bmax)+1)/10.0, 0.1)
            m = []
            for i, b in enumerate(self.losses['B']):
                pfunc = ip.interp1d(b, self.losses['pfe'][i], kind='cubic')
                bx = [x for x in Bv if x < b[-1]]
                m.append(pfunc(bx).tolist() + (len(Bv)-len(bx))*[None])
            self.losses['B'] = Bv
            self.losses['pfe'] = list(zip(*m))

    def getValues(self):
            
        return {
            'name': self.name,
            'desc': self.mc1_title,
            'cversion': self.version_mc_curve,
            'ctype': self.mc1_type,
            'recalc': self.mc1_recalc,
            'remz': self.mc1_remz,
            'bsat': self.mc1_bsat,
            'bref': self.mc1_bref,
            'fillfac': self.mc1_fillfac,
            'curve': self.curve,
            'fo': self.fo,
            'Bo': self.Bo,
            'ch': self.ch,
            'ch_freq': self.ch_freq,
            'cw': self.cw,
            'cw_freq': self.cw_freq,
            'b_coeff': self.b_coeff,
            'rho': self.rho,
            'fe_sat_mag': self.fe_sat_mag,
            'losses': self.losses}
                                     
if __name__ == "__main__":
    import matplotlib.pylab as pl
    if len(sys.argv) == 2:
        filename = sys.argv[1]
    else:
        filename = sys.stdin.readline().strip()
            
    tks = Reader(filename)
    
    if tks.losses:
        import numpy as np
        cw = tks.cw
        alpha = tks.cw_freq
        ch = tks.ch
        beta = tks.ch_freq
        gamma = tks.b_coeff
        
        for i, f in enumerate(tks.losses['f']):
            pfe = [p for p in np.array(tks.losses['pfe']).T[i] if p]
            pl.plot(tks.losses['B'], pfe1(f, np.array(tks.losses['B']),
                                          cw, alpha, ch, beta, gamma))
            pl.plot(tks. losses['B'][:len(pfe)], pfe,
                    marker='o', label="{} Hz".format(f))

        pl.title("Iron Losses " + filename)
        #pl.yscale('log')
        #pl.xscale('log')
        pl.xlabel("Induction [T]")
        pl.ylabel("Pfe [W/kg]")
        #pl.legend()
        pl.grid(True)
        #pl.savefig('tks.png')
        pl.show()

    mcv = tks.getValues()
    json.dump(mcv, sys.stdout)



# -*- coding: utf-8 -*-
"""
    femagtools.machine
    ~~~~~~~~~~~~~~~~~~

    Calculate operating charactistics of a pm/rel machine

    :copyright: 2016 Semafor Informatik & Energie AG, Basel
    :license: BSD, see LICENSE for more details.
"""
import sys
import numpy as np
import numpy.linalg as la
import scipy.optimize as so
import scipy.interpolate as ip
import logging

logger = logging.getLogger(__name__)


def i1beta(iq, id):
    return (np.arctan2(id, iq),
            la.norm(np.array((iq, id))))  # /np.sqrt(2.0)


def idq(i1, beta):
    return i1*np.array((np.sin(beta),
                        np.cos(beta)))  # *np.sqrt(2.0)


class PmRelMachine(object):
    """Abstract base class for PmRelMachines

    ::param m: number of winding phases
    ::param p: number of pole pairs
    ::param r1: stator winding resistance (in Ohm)
    """
    def __init__(self, m, p, r1):
        self.p = p
        self.m = m
        self.r1 = r1

    def iqd_torque(self, torque):
        """minimum d-q-current for torque"""
        res = so.minimize(lambda idq: la.norm(idq), (0, 0), method='SLSQP',
                          constraints=({'type': 'eq',
                                        'fun': lambda iqd:
                                        self.torque_iqd(*iqd) - torque}))
        return res.x

    def iqd_torque_umax(self, torque, w1, u1max):
        "d-q current and torque at stator frequency and max voltage"
        iq, id = self.iqd_torque(torque)
        #sys.stderr.write( "id {} iq {} torque {}\n".format(id,iq,torque ))
        # check voltage
        if la.norm(self.uqd(w1, iq, id)) <= u1max:
            return (iq, id)
        # decrease psi (flux weakening mode)
        return so.fsolve(lambda iqd: (la.norm(self.uqd(w1, *iqd)) - u1max,
                                      self.torque_iqd(*iqd)-torque), (iq, id))
        
    def beta_u(self, w1, u, i1):
        "beta at given frequency, voltage and current"
        return so.fsolve(lambda b:
                         la.norm(self.uqd(w1, *(idq(i1, b[0])[::-1])))-u,
                         -np.pi/3)[0]

    def i1_u(self, w1, u, beta):
        "i1 at given frequency, voltage and beta angle"
        return so.fsolve(lambda i1:
                         la.norm(self.uqd(w1, *(idq(i1, beta))[::-1]))-u,
                         0)[0]
    
    def iq_u(self, w1, u, id):
        "iq at given frequency, voltage and id current"
        i0 = np.sqrt(((u/w1)**2 - self.psid(0, id)**2) / self.lq(id, 0))
        return so.fsolve(lambda iq:
                         la.norm(self.uqd(w1, iq, id))-u, i0)[0]
    
    def id_u(self, w1, u, iq):
        "id at given frequency, voltage and iq current"
        i0 = np.sqrt(((u/w1)**2 - self.psim(0, iq) -
                      self.psiq(iq, 0)**2) / self.ld(0, iq))
        return so.fsolve(lambda id:
                         la.norm(self.uqd(w1, iq, id))-u, i0)[0]

    def w1_u(self, u, id, iq):
        "w1 at given voltage and id,iq current"
        w10 = u/np.sqrt(self.psid(iq, id)**2 + self.psiq(iq, id)**2)
        return so.fsolve(lambda w1:
                         la.norm(self.uqd(w1, iq, id))-u, w10)[0]

    def iq_torque(self, torque, id):
        "q current with given torque and d-current"
        i0 = self.i0 if torque > 0 else - self.i0
        return so.fsolve(lambda iq: self.torque_iqd(iq, id)-torque,
                         i0)[0]
    
    def id_torque(self, torque, iq):
        "d current with given torque and d-current"
        i0 = self.i0 if torque > 0 else - self.i0
        return so.fsolve(lambda id: self.torque_iqd(iq, id)-torque, i0)[0]
    
    def i1_torque(self, torque, beta):
        "i1 current with given torque and beta"
        i0 = 0  # torque/(self.m*self.p*self.psid(beta, 0))
        return so.fsolve(lambda i1: self.torque(beta, i1)-torque, i0)[0]

    def tfric(self, wm):
        return 0.0
    
    def characteristics(self, torque_list, n_list, u1):
        r = dict(id=[], iq=[], uq=[], ud=[], u1=[], i1=[], T=[],
                 beta=[], gamma=[], phi=[], cosphi=[], pmech=[], n=[])
        for torque, n in zip(torque_list, n_list):
            w1 = 2*np.pi*n*self.p
            iq, id = self.iqd_torque_umax(torque, w1, u1)
            #print( 'p={} w1={} tq={} id={} iq={} u1={}'.format(self.p,w1,torque,id,iq,u1) )
            r['id'].append(id)
            r['iq'].append(iq)
            uq,ud=self.uqd(w1, iq, id)
            #print( 'uq={} ud={}'.format(uq, ud) )
            r['uq'].append(uq)
            r['ud'].append(ud)
            r['u1'].append(np.sqrt(ud**2+uq**2))  # /np.sqrt(2.0))
            r['i1'].append(np.sqrt(id**2+iq**2))  # /np.sqrt(2.0))
            tq = self.torque_iqd(iq, id)
            r['T'].append(tq)
            r['beta'].append(np.arctan2(id, iq)/np.pi*180.)
            r['gamma'].append(np.arctan2(ud, uq)/np.pi*180.)

            r['n'].append(n)
            r['phi'].append(r['beta'][-1] - r['gamma'][-1])
            r['cosphi'].append(np.cos(r['phi'][-1]/180*np.pi))
            r['pmech'].append(w1/self.p*r['T'][-1])
        return r


class PmRelMachineLdq(PmRelMachine):
    """Standard set of PM machine given by i1,beta parameters:
    p number of pole pairs
    m number of phases
    psim flux in Vs (RMS)
    ld d-inductance in H
    lq q-inductance in H
    r1 stator resistance
    beta angle i1 vs up in degrees
    i1 current in A (RMS)
    """
    def __init__(self, m, p, psim, ld, lq, r1, beta=None, i1=None):
        super(self.__class__, self).__init__(m, p, r1)
        # start value for torque iterations
        self.i0 = np.max(i1)/2 if i1 else 0.0
        if isinstance(ld, (float, int)):
            self.ld = lambda b, i: ld
            self.lq = lambda b, i: lq
            self.psim = lambda b, i: psim
            logger.debug("ld %s lq %s psim %s", ld, lq, psim)
            return
        if len(beta) < 2:
            if isinstance(ld[0], list):  # TODO check this
                self.ld = lambda b, i: ld[-1][-1]
                self.lq = lambda b, i: lq[-1][-1]
                self.psim = lambda b, i: psim[-1][-1]
            else:
                self.ld = lambda b, i: ld[-1]
                self.lq = lambda b, i: lq[-1]
                self.psim = lambda b, i: psim[-1]
            logger.debug("ld %s lq %s psim %s", self.ld(beta[-1], 0),
                         self.lq(beta[-1], 0), self.psim(beta[-1], 0))
            return

        beta = np.asarray(beta)*np.pi/180.
        i1 = np.asarray(i1)
        ld = np.asarray(ld)
        lq = np.asarray(lq)
        psim = np.asarray(psim)

        if len(i1) < 4 or len(beta) < 4:
            if len(i1) == len(beta):
                self._ld = ip.interp2d(beta, i1, ld.T)
                self._lq = ip.interp2d(beta, i1, lq.T)
                self._psim = ip.interp2d(beta, i1, psim.T)
                self.ld = lambda x, y: self._ld(x, y)[0]
                self.lq = lambda x, y: self._lq(x, y)[0]
                self.psim = lambda x, y: self._psim(x, y)[0]
                logger.debug("interp2d beta %s i1 %s", beta, i1)
                return
            elif len(i1) == 1:
                self._ld = ip.InterpolatedUnivariateSpline(beta, ld, k=1)
                self._lq = ip.InterpolatedUnivariateSpline(beta, lq, k=1)
                self._psim = ip.InterpolatedUnivariateSpline(beta, psim, k=1)
                self.ld = lambda x, y: self._ld(x)
                self.lq = lambda x, y: self._lq(x)
                self.psim = lambda x, y: self._psim(x)
                logger.debug("interpolatedunivariatespline beta %s", beta)
                return
            if len(beta) == 1:
                self._ld = ip.InterpolatedUnivariateSpline(i1, ld, k=1)
                self._lq = ip.InterpolatedUnivariateSpline(i1, lq, k=1)
                self._psim = ip.InterpolatedUnivariateSpline(i1, psim, k=1)
                self.ld = lambda x, y: self._ld(y)
                self.lq = lambda x, y: self._lq(y)
                self.psim = lambda x, y: self._psim(y)
                logger.debug("interpolatedunivariatespline i1 %s", i1)
                return
            
            raise ValueError("unsupported array size {}x{}".format(len(beta), len(i1)))
            
        self._ld = ip.RectBivariateSpline(beta, i1, ld)
        self._lq = ip.RectBivariateSpline(beta, i1, lq)
        self._psim = ip.RectBivariateSpline(beta, i1, psim)
        self.ld = lambda x, y: self._ld(x, y)[0][0]
        self.lq = lambda x, y: self._lq(x, y)[0][0]
        self.psim = lambda x, y: self._psim(x, y)[0][0]
        logger.debug("rectbivariatespline beta %s i1 %s", beta, i1)
        
    def z(self, w1, beta, i1):
        "machine impedance in d-q reference frame"
        return np.array(((self.r1, w1*self.ld(beta, i1)),
                         (-w1*self.lq(beta, i1), self.r1)))
    
    def uqd(self, w1, iq, id):
        "stator voltage at freq and stator current in dq components"
        beta, i1 = i1beta(iq, id)
        return self.z(w1, beta, i1). dot(np.array((iq, id))) + \
            np.array((w1 * self.psim(beta, i1), 0))
        
    def ui1beta(self, w1, i1, beta):
        "stator voltage at freq and stator current in ampl and phase"
        id, iq = idq(i1, beta)
        return self.z(w1, beta, i1).dot(np.array((iq, id))) + \
            np.array((w1 * self.psim(beta, i1), 0))
        
    def torque_iqd(self, iq, id):
        "torque at q-d-current"
        beta, i1 = i1beta(iq, id)
        psid = self.ld(beta, i1)*id + self.psim(beta, i1)
        psiq = self.lq(beta, i1)*iq
        #sys.stderr.write( 'i1 {} beta {} id {} iq {} ld {} lq {}\n'.format(
        #    i1, beta, id, iq, self.ld(beta,i1), self.lq(beta,i1) ) )
        return self.m*self.p*(psid*iq - psiq*id)

    def torque(self, beta, i1):
        "torque at i1-beta-current"
        id, iq = idq(i1, beta)
        psid = self.ld(beta, i1)*id + self.psim(beta, i1)
        psiq = self.lq(beta, i1)*iq
        return self.m*self.p*(psid*iq - psiq*id)
    
    def psid(self, beta, i1):
        "psid at i1-beta-current"
        id, iq = idq(i1, beta)
        return self.ld(beta, i1)*id + self.psim(beta, i1)

    def psiq(self, beta, i1):
        "psiq at i1-beta-current"
        id, iq = idq(i1, beta)
        return self.lq(beta, i1)*iq

    def i1beta_characteristics(self, n_list, i1_list, beta_list, u1max):
        r = dict(id=[], iq=[], uq=[], ud=[], u1=[], i1=[], T=[],
                 beta=[], gamma=[], phi=[], cosphi=[], pmech=[], n=[])
        for n, i1, beta in zip(n_list, i1_list, beta_list):
            w1 = 2*np.pi*n*self.p
            beta = beta/180*np.pi
            id, iq = idq(i1, beta)
            uq, ud = self.uqd(w1, iq, id)
            u1 = np.sqrt(ud**2+uq**2)
            if u1 > u1max:
                logger.debug("u1 %s > %s", u1, u1max)
                beta = self.beta_u(w1, u1max, i1)
                logger.debug("beta %s", beta*180/np.pi)
                id, iq = idq(i1, beta)
                logger.debug("beta %s id, %s iq %s", beta*180/np.pi, id, iq)
                uq, ud = self.uqd(w1, iq, id)
                u1 = np.sqrt(ud**2+uq**2)
                logger.debug("ud %s uq %s --> u1 %s", ud, uq, u1)
                
            tq = self.torque(beta, i1)
            #print( 'p={} w1={} tq={} id={} iq={} u1={}'.format(self.p,w1,torque,id,iq,u1) )
            r['id'].append(id)
            r['iq'].append(iq)

            #print( 'uq={} ud={}'.format(uq, ud) )
            r['uq'].append(uq)
            r['ud'].append(ud)
            r['u1'].append(u1)  # /np.sqrt(2.0))
            r['i1'].append(np.sqrt(id**2+iq**2))  # /np.sqrt(2.0))
            tq = self.torque_iqd(iq, id)
            r['T'].append(tq)
            r['beta'].append(np.arctan2(id, iq)/np.pi*180.)
            r['gamma'].append(np.arctan2(ud, uq)/np.pi*180.)

            r['n'].append(n)
            r['phi'].append(r['beta'][-1]-r['gamma'][-1])
            r['cosphi'].append(np.cos(r['phi'][-1]/180*np.pi))
            r['pmech'].append(w1/self.p*r['T'][-1])
        return r


class PmRelMachinePsidq(PmRelMachine):
    """Standard set of PM machine parameters:
    p number of pole pairs
    m number of phases

    psid d-flux (Vs Peak)
    psiq q-flux (Vs Peak)
    r1 stator resistance
    id q current (A, Peak)
    iq q current (A, Peak)
    """

    def __init__(self, m, p, psid, psiq, r1, id, iq):
        super(self.__class__, self).__init__(m, p, r1)

        if isinstance(psid, (float, int)):
            self.psid = lambda id, iq: psid / np.sqrt(2.0)
            self.psiq = lambda id, iq: psiq / np.sqrt(2.0)
            self.i0 = 1.0  # start value for iterations
            return

        self.i0 = np.max(iq)/2  # start value for iterations
        
        psid = np.asarray(psid) / np.sqrt(2.0)
        psiq = np.asarray(psiq) / np.sqrt(2.0)
        id = np.asarray(id) / np.sqrt(2.0)
        iq = np.asarray(iq) / np.sqrt(2.0)
        
        if len(iq) < 4 or len(id) < 4:
            self._psid = ip.interp2d(iq, id, psid.T)
            self._psiq = ip.interp2d(iq, id, psiq.T)
            self.psid = lambda id, iq: self._psid(iq, id)[0]
            self.psiq = lambda id, iq: self._psiq(iq, id)[0]

        else:
            self._psid = ip.RectBivariateSpline(iq, id, psid)
            self._psiq = ip.RectBivariateSpline(iq, id, psiq)
            self.psid = lambda id, iq: self._psid(iq, id)[0][0]
            self.psiq = lambda id, iq: self._psiq(iq, id)[0][0]
        
    def z(self, w1):
        return np.array([[self.r1, 0],
                         [0, self.r1]])
    
    def uqd(self, w1, iq, id):
        "stator voltage at freq and stator current"
        return (self.z(w1).dot(np.array([iq, id])) +
                np.array([[0, w1],
                          [-w1, 0]]).dot(np.array(
                              [[self.psiq(id, iq)],
                               [self.psid(id, iq)]]))).T[0]
        
    def torque_iqd(self, iq, id):
        "torque at q-d-current"
        #sys.stderr.write( 'id {} iq {} psid {} psiq {}\n'.format(
        #    id, iq, self.psid(id,iq), self.psiq(id,iq) ) )
        return self.m*self.p*(self.psid(id, iq)*iq - self.psiq(id, iq)*id)
    
    def ld(self, id, iq):
        "ld at d-q-current"
        return (self.psid(id, iq) - self.psim(id, iq))/id

    def lq(self, id, iq):
        "lq at d-q-current"
        return self.psiq(id, iq)/iq

    def psim(self, id, iq):
        "psim at d-q-current"
        return self.psid(0, iq)

    def torque(self, beta, i1):
        "torque at i1-beta-current"
        id, iq = idq(i1, beta)
        return self.torque_iqd(iq, id)

def main(m, op):
#if all (k in m for k in ('psim','u1')) and \
#    all (k in op for k in ('T','n')):
    r10=m.get('r1',0)
    if 'beta' in m:
        pm = PmRelMachineLdq(3, m['p'],
                             np.array(m['psim']),
                             np.array(m['ld']),
                             np.array(m['lq']),
                             r10,
                             np.array(m['beta']),
                             np.array(m['i1']))
    else:
        pm = PmRelMachinePsidq(3, m['p'],
                               np.array(m['psid']),
                               np.array(m['psiq']),
                               np.array(m['torque']),
                               m['r1'],
                               np.array(m['Id']),
                               np.array(m['iq']))
            
    return pm.characteristics(op['T'], op['n'], m['u1'])

    return {}

if __name__ == "__main__":
    import json
    m = json.load(sys.stdin)

    r = {}
    r = main(m['machine'], m['op'])
    json.dump(r, sys.stdout)
    sys.exit(0)



# -*- coding: utf-8 -*-
"""
    femagtools.model
    ~~~~~~~~~~~~~~~~

    Managing model parameters

    :copyright: 2016 Semafor Informatik & Energie AG, Basel
    :license: BSD, see LICENSE for more details.
"""
import logging

logger = logging.getLogger(__name__)


def gcd(a, b):
    while b:
        a, b = b, a % b
    return a


def lcm(a, b):
    if a or b:
        return abs(a*b)/gcd(a, b)
    return 0


class MCerror(Exception):
    pass


class Model(object):
    def __init__(self, parameters):
        for k in parameters.keys():
            setattr(self, k, parameters[k])
            
    def set_value(self, name, value, p=None):
        """set value of parameter identified by name
        
        :param name: name of parameter
        :param value: value to be assigned to parameter
        """
        if isinstance(name, str):
            setattr(self, name, value)
            return
        
        if len(name) > 1:
            k = name[0]
            if hasattr(self, k):
                self.set_value(name[1:], value, getattr(self, k))
                return
            elif p:
                self.set_value(name[1:], value, p[k])
                return
            self.set_value(name[1:], value, self)
            return
        if p:
            p[name[0]] = value
            return
        setattr(self, name[0], value)
        
    def get(self, name, r=None):
        """return value of key name

        :param name: key of parameter value
        :returns: value of parameter identified by key
        """        
        try:
            if isinstance(name, str):
                return getattr(self, name)
            if r and type(r) == dict:
                for k in name:
                    r = r.get(k)
                return r
            if len(name) > 1:
                if r:
                    return self.get(name[1:], getattr(r, name[0]))
                return self.get(name[1:], getattr(self, name[0]))
            return getattr(self, name[0])
        except KeyError as e:
            logger.error(e)
            raise MCerror(e)
     
    def __str__(self):
        "return string format of this object"
        return repr(self.__dict__)

    def __repr__(self):
        "representation of this object"
        return self.__str__()


class MachineModel(Model):
    """represents a machine model for a FE analysis
    
    :param parameters: dict containing the model parameters. For example:
    ::

        {'lfe': 0.1,
        'poles': 4,
        'outer_diam': 0.13,
        'bore_diam': 0.07,
        'stator':{
        'num_slots': 12,
        'num_slots_gen': 3,
        ...
        },
        'magnet':{
            'material': 'M395',
            ..
        }
        }

    """
    def __init__(self, parameters):
        super(self.__class__, self).__init__(parameters)

        if 'name' in parameters:
            # must replace white space
            name = parameters['name'].strip().replace(' ', '_')
            for c in ('"', '(', ')'):
                name = name.replace(c, '')
            setattr(self, 'name', name)
        else:
            setattr(self, 'name', 'DRAFT')
        try:
            self.external_rotor = (self.external_rotor == 1)
        except:
            self.external_rotor = False
        self.move_inside = 1.0 if self.external_rotor else 0.0
        if 'magnet' in parameters:
            if 'mcvkey_mshaft' in self.magnet:
                self.magnet['mcvkey_shaft'] = self.magnet['mcvkey_mshaft']
            for mcv in ('mcvkey_yoke', 'mcvkey_shaft'):
                if mcv not in self.magnet:
                    self.magnet[mcv] = 'dummy'
        if 'coord_system' in parameters:
            self.move_action = 1
        else:
            self.coord_system = 0
            self.move_action = 0

        if 'stator' in parameters and 'num_slots_gen' not in self.stator:
            try:
                m = self.windings['num_phases']
            except:
                m = 1

            try:
                self.stator['num_slots_gen'] = (m*self.stator['num_slots'] /
                                                gcd(self.stator['num_slots'],
                                                    m*self.poles))
            except:
                pass
            
    def set_magcurves(self, magcurves):
        """set and return real names of magnetizing curve material

        :param magcurves: :class: 'MagnetizingCurve' including
                              magnetizing curve materials

        :returns: set of magnetizing curve names attached to this model

        """
        names = []
        missing = []
        if magcurves:
            if 'stator' in self.__dict__:
                try:
                    if self.stator['mcvkey_yoke'] != 'dummy':
                        mcv = magcurves.find(self.stator['mcvkey_yoke'])
                        if mcv:
                            logger.debug('stator mcv %s', mcv)
                            self.stator['mcvkey_yoke'] = mcv
                            names.append(mcv)
                        else:
                            missing.append(self.stator['mcvkey_yoke'])
                            logger.error('stator mcv %s not found',
                                         self.stator['mcvkey_yoke'])
                except KeyError:
                    pass
                
            if 'magnet' in self.__dict__:
                try:
                    if self.magnet['mcvkey_yoke'] != 'dummy':
                        mcv = magcurves.find(self.magnet['mcvkey_yoke'])
                        if mcv:
                            logger.debug('magnet mcv %s', mcv)
                            self.magnet['mcvkey_yoke'] = mcv
                            names.append(mcv)
                        else:
                            missing.append(self.magnet['mcvkey_yoke'])
                            logger.error('magnet mcv %s not found',
                                         self.magnet['mcvkey_yoke'])
                except KeyError:
                    pass

                try:
                    if self.magnet['mcvkey_shaft'] != 'dummy':
                        mcv = magcurves.find(self.magnet['mcvkey_shaft'])
                        if mcv:
                            logger.debug('shaft mcv %s', mcv)
                            self.magnet['mcvkey_shaft'] = mcv
                            names.append(mcv)
                        else:
                            missing.append(self.magnet['mcvkey_shaft'])
                            logger.error('magnet shaft %s not found',
                                         self.magnet['mcvkey_shaft'])
                except KeyError:
                    pass
        if missing:
            raise MCerror("MC pars missing: {}".format(
                ', '.join(set(missing))))
        return set(names)
    
    def statortype(self):
        """return type of stator slot"""
        for k in self.stator:
            if isinstance(self.stator[k], dict):
                return k
        raise MCerror("Missing stator slot model in {}".format(self.stator))
    
    def magnettype(self):
        """return type of magnet slot"""
        for k in self.magnet:
            if isinstance(self.magnet[k], dict):
                return k
        raise MCerror("Missing magnet model in {}".format(self.magnet))

    def is_complete(self):
        """check completeness of models"""
        try:
            self.statortype()
            self.magnettype()
            return True
        except:
            return False
        

class FeaModel(Model):
    def __init__(self, parameters):
        self.cufilfact = 0.45
        self.culength = 1.4
        self.wind_temp = 20
        self.slot_indul = 0.0
        self.skew_angle = 0.0
        self.num_skew_steps = 0
        self.num_par_wdgs = 1
        self.eval_force = 0
        self.optim_i_up = 0
        self.plots = []
        super(self.__class__, self).__init__(parameters)
        
    def __getitem__(self, k, default=None):
        if default:
            return getattr(self, k, default)
        return getattr(self, k)



"""
    femagtools.vbf
    ~~~~~~~~~~~~~~

    Manage VBF magnetizing curve data files

    :copyright: 2016 Semafor Informatik & Energie AG, Basel
    :license: BSD, see LICENSE for more details.
"""
import sys
import numpy as np
import femagtools.losscoeffs


fo = 50.
Bo = 1.5


def pfe1(f, B, ch, fh, cw, fw, fb):
    return (ch*(f/fo)**fh + cw*(f/fo)**fw)*(B/Bo)**fb


def pfe2(f, B, cw, fw, fb):
    return cw*(f/fo)**fw * (B/Bo)**fb


def logpfe2(f, B, cw, fw, fb):
    return np.log10(cw) + fw*np.log10(f/fo) * fb*np.log10(B/Bo)


class Reader:

    def __init__(self, filename):
        self.vbf = {}
        with open(filename) as f:
            self.vbf['name'] = f.readline().strip()
            self.vbf['fo'], self.vbf['Bo'] = [float(s)
                                              for s in f.readline().strip().split()]
            # Ignore the next line
            f.readline()
            self.vbf['f'] = [float(s) for s in f.readline().strip().split()]
            self.vbf['B'] = []
            self.vbf['pfe'] = []
            for l in f.readlines():
                values = [float(s) for s in l.strip().split()]
                if len(values) > 1:
                    self.vbf['B'].append(values[0])
                    self.vbf['pfe'].append(
                        [v if v > 0 else None for v in values[1:]])
            
    def getLossValues(self):
        return self.vbf
    
if __name__ == "__main__":
    import matplotlib.pylab as pl
    if len(sys.argv) == 2:
        filename = sys.argv[1]
    else:
        filename = sys.stdin.readline().strip()
            
    vbf = Reader(filename)

    print(vbf.getLossValues())
    
#    cw, alfa, beta = femagtools.losscoeffs.fitsteinmetz(
#        vbf.vbf['f'],
#        vbf.vbf['B'],
#        vbf.vbf['pfe'],
#        vbf.vbf['Bo'],
#        vbf.vbf['fo'])

    n = 100
    B = pl.np.linspace(0.1, 2, n)
    
#    for i in range(len(vbf.vbf['f'])):
#        f = vbf.vbf['f'][i]
#        pfe = [p for p in np.array(vbf.vbf['pfe']).T[i] if p]
#        pl.plot(B, pfe2(f, B, cw, alfa, beta))
#        pl.plot(vbf. vbf['B'][:len(pfe)], pfe,
#                marker='o', label="f1={} Hz".format(f))

#    pl.title("Iron Losses " + filename)
#    pl.yscale('log')
#    pl.xscale('log')
#    pl.xlabel("Induction [T]")
#    pl.ylabel("Pfe [W/kg]")
#    pl.grid(True)
#    pl.show()

    z = femagtools.losscoeffs.fitjordan2(
        vbf.vbf['f'],
        vbf.vbf['B'],
        vbf.vbf['pfe'],
        vbf.vbf['Bo'],
        vbf.vbf['fo'])

    cw, alpha, ch, beta, gamma = z
#    cw, alpha, beta = z
#    print(cw, alpha, beta)
#    cw = 2.89
#    alpha = 1.43
#    beta = 1.85
    for i, f in enumerate(vbf.vbf['f']):
        pfe = [p for p in np.array(vbf.vbf['pfe']).T[i] if p]
        pl.plot(B, pfe1(f, B, cw, alpha, ch, beta, gamma))
        pl.plot(vbf. vbf['B'][:len(pfe)], pfe,
                marker='o', label="{} Hz".format(f))

    pl.title("Iron Losses " + filename)
    pl.yscale('log')
    pl.xscale('log')
    pl.xlabel("Induction [T]")
    pl.ylabel("Pfe [W/kg]")
    pl.legend()
    pl.grid(True)
    pl.show()



"""
    femagtools.fsl
    ~~~~~~~~~~~~~~

    Creating FSL Scripts

    :copyright: 2016 Semafor Informatik & Energie AG, Basel
    :license: BSD, see LICENSE for more details.
"""
import logging
import mako
import mako.lookup
import os
import re

logger = logging.getLogger(__name__)


class FslBuilderError(Exception):
    pass


class Builder:
    def __init__(self):
        self.lookup = mako.lookup.TemplateLookup(
            directories=[os.path.join(os.path.dirname(__file__), 'templates'),
                         os.path.join(os.getcwd(), 'templates')],
            disable_unicode=False, input_encoding='utf-8',
            output_encoding='utf-8',
            default_filters=['decode.utf8'])
    
    def create_stator_model(self, model):
        templ = model.statortype()
        return self.__render(model, templ)
    
    def create_magnet_model(self, model):
        mcv = ["mcvkey_yoke='{}'".format(
            model.magnet.get('mcvkey_yoke', 'dummy')),
               "mcvkey_shaft='{}'".format(
                   model.magnet.get('mcvkey_shaft', 'dummy'))]
        try:
            if 'magnetFsl' in model.magnet:
                if 'parameter' in model.magnet['magnetFsl']:
                    return mcv + self.render_template(
                        model.magnet['magnetFsl']['content_template'],
                        model.magnet['magnetFsl']['parameter'])
                if isinstance(model.magnet['magnetFsl']
                              ['content_template'], str):
                    with open(model.magnet['magnetFsl']
                              ['content_template']) as f:
                        templ = [l.strip() for l in f.readlines()]
                else:
                    templ = model.magnet['magnetFsl']['content_template']
                logger.info(model.magnet['magnetFsl'])
                return mcv + self.render_template(
                    '\n'.join(templ),
                    model.magnet['magnetFsl'])

            templ = model.magnettype()
            return mcv + self.__render(model, templ)
        except AttributeError:
            pass  # no magnet
        return []

    def create_connect_models(self, model):
        """return connect_model if rotating machine"""
        if model.get('move_action') == 0:
            return ['pre_models("connect_models")']
        return []
    
    def create_open(self, model):
        return self.__render(model, 'open') + \
            self.__render(model, 'basic_modpar')

    def set_modpar(self, model):
        return self.__render(model, 'basic_modpar')

    def create_new_model(self, model):
        return self.__render(model, 'new_model')
    
    def create_model(self, model, magnets=None):
        return self.create_new_model(model) + \
            self.__render(model.windings, 'cu_losses') + \
            self.create_stator_model(model) + \
            ['post_models("nodedistance", "ndst" )',
             'agndst=ndst[1]*1e3'] + \
            self.__render(model, 'gen_winding') + \
            self.create_magnet(model, magnets) + \
            self.create_magnet_model(model) + \
            self.create_connect_models(model) + \
            ['']
    
    def open_model(self, model, magnets=None):
        return self.create_open(model) + \
            self.create_magnet(model, magnets) + \
            self.__render(model.windings, 'cu_losses')
    
    def load_model(self, model, magnets=None):
        return self.__render(model, 'open')
    
    def create_magnet(self, model, magnets):
        try:
            if magnets and 'material' in model.magnet:
                magnet = magnets.find(model.magnet['material'])
                if magnet:
                    return self.__render(magnet, 'magnet')
                raise FslBuilderError('magnet material {} not found'.format(
                    model.magnet['material']))
            return [' m.remanenc       =  1.2',
                    ' m.relperm        =  1.05']
        except AttributeError:
            pass  # no magnet
        return []

    def create_common(self, model):
        return self.__render(model, 'common')
    
    def create_analysis(self, model):
        if model['calculationMode'] in ('cogg_calc',
                                        'ld_lq_fast',
                                        'pm_sym_loss',
                                        'psd_psq_fast'):
            return self.__render(model, model['calculationMode'])
        return (self.__render(model, 'cu_losses') + 
                self.__render(model, model['calculationMode']) +
                self.__render(model, 'plots'))
            
    def create_airgap_induc(self, model):
            return self.__render(model, 'airgapinduc')
        
    def create_colorgrad(self, model):
            return self.__render(model, 'colorgrad')

    def create(self, model, fea, magnets=None):
        "create model and analysis function"
        fea['lfe'] = model.get('lfe')
        fea['move_action'] = model.get('move_action')
        fea.update(model.windings)
        if model.is_complete():
            return self.create_model(model, magnets) + \
                self.create_analysis(fea)
        return self.open_model(model, magnets) + \
            self.create_analysis(fea)
        
    def __render(self, model, templ):
        template = self.lookup.get_template(templ+".mako")
        return template.render_unicode(model=model).split('\n')

    def render_template(self, content_template, parameters):
        template = mako.template.Template(content_template)
        obj = {}
        if isinstance(parameters, list):
            for p in parameters:
                obj[p['key']] = p['value']
            return template.render_unicode(
                par=obj).split('\n')
        return template.render_unicode(
            model=parameters).split('\n')
    
    def read(self, fslfile):
        """extracts parameters from content and creates template"""
        parpat = re.compile(
            r'''(\w+)\s*= # key
            \s*(([+-]?\d+((?:\.\d+)?(?:[eE][+-]\d+)?))|
            (['"][^'"]*['"]))\s*-- # value
            \s*(.+)$''', re.X)
        content = []
        content_template = []
        parameter = []
        for line in fslfile:
            # add line to content
            content.append(line.strip())
            p = parpat.findall(line)
            if p:
                par = dict(key=p[0][0],
                           value=p[0][1].strip(),
                           comment=p[0][-1])
                parameter.append(par)
                content_template.append("{0} = {1} -- {2}".format(
                    par['key'],
                    '${par[\''+par['key']+'\']}',
                    par['comment']))
                continue

            # nothing do do => insert this line in template
            content_template.append(line.strip())

        return dict(
            content='\n'.join(content),
            type='fsl',
            content_template='\n'.join(content_template),
            parameter=parameter)

